@@ -1,7 +1,9 @@
 import io
+
 import json
 import os
 import unittest
+
 import unittest.mock
 from pkg_resources import resource_filename, resource_string
 from xmlrpc import client as xmlrpclib
@@ -16,12 +18,12 @@ if not isinstance(long_description, str):
 long_description = io.StringIO(long_description, newline=None).read()
 
 COMPLETE = {
-    "metadata_version": "2.1",
+    "metadata-version": "2.4",
     "name": "complete",
     "version": "1.0.dev1",
-    "description": "This is a test package for pyroma.",
-    "long_description": long_description,
-    "classifiers": [
+    "summary": "This is a test package for pyroma.",
+    "description": long_description,
+    "classifier": [
         "Development Status :: 6 - Mature",
         "Operating System :: OS Independent",
         "Programming Language :: Python :: 2.6",
@@ -31,13 +33,26 @@ COMPLETE = {
         "Programming Language :: Python :: 3.3",
         "License :: OSI Approved :: MIT License",
     ],
+    "dynamic": [
+        "author",
+        "author-email",
+        "classifier",
+        "description",
+        "home-page",
+        "keywords",
+        "license",
+        "project-url",
+        "requires-dist",
+        "requires-python",
+        "summary",
+    ],
     "keywords": "pypi,quality,example",
     "author": "Lennart Regebro",
-    "author_email": "regebro@gmail.com",
-    "url": "https://github.com/regebro/pyroma",
-    "project_urls": "Source Code, https://github.com/regebro/pyroma",
-    "requires_dist": "zope.event",
-    "python_requires": ">=2.6",
+    "author-email": "regebro@gmail.com",
+    "home-page": "https://github.com/regebro/pyroma",
+    "project-url": "Source Code, https://github.com/regebro/pyroma",
+    "requires-dist": "zope.event",
+    "requires-python": ">=2.6",
     "license": "MIT",
 }
 
@@ -100,13 +115,12 @@ proxystub = ProxyStub()
 
 
 class RatingsTest(unittest.TestCase):
-
     maxDiff = None
 
-    def _get_file_rating(self, dirname):
+    def _get_file_rating(self, dirname, skip_tests=None):
         directory = resource_filename(__name__, os.path.join("testdata", dirname))
         data = projectdata.get_data(directory)
-        return rate(data)
+        return rate(data, skip_tests)
 
     def test_complete(self):
         rating = self._get_file_rating("complete")
@@ -115,32 +129,58 @@ class RatingsTest(unittest.TestCase):
 
     def test_setup_config(self):
         rating = self._get_file_rating("setup_config")
-        self.assertEqual(rating, (10, []))
+        self.assertEqual(
+            rating,
+            (
+                9,
+                [
+                    "Your project does not have a pyproject.toml file, which is highly recommended.\n"
+                    "You probably want to create one with the following configuration:\n\n"
+                    "    [build-system]\n"
+                    '    requires = ["setuptools>=42"]\n'
+                    '    build-backend = "setuptools.build_meta"\n',
+                ],
+            ),
+        )
 
     def test_only_config(self):
+        # In version 5, this is now an error, as there is no legacy setup.py,
+        # nor a modern pyproject.toml.
         rating = self._get_file_rating("only_config")
 
         self.assertEqual(
             rating,
             (
-                7,
+                5,
                 [
-                    (
-                        "You should specify what Python versions you support with "
-                        "the 'requires-python'/'python_requires' metadata."
-                    ),
-                    "You seem to have a setup.cfg, but neither a setup.py, nor a build-system defined. "
-                    "This makes it unclear how your project should be built. You probably want to "
-                    "have a pyproject.toml file with the following configuration:\n\n"
+                    "You seem to neither have a setup.py, nor a pyproject.toml, only setup.cfg.\n"
+                    "This makes it unclear how your project should be built, and some packaging "
+                    "tools may fail.",
+                    "Your project does not have a pyproject.toml file, which is highly "
+                    "recommended.\n"
+                    "You probably want to create one with the following configuration:\n\n"
                     "    [build-system]\n"
                     '    requires = ["setuptools>=42"]\n'
-                    '    build-backend = "setuptools.build_meta"\n\n'
-                    "In the future this will become a hard failure and your package will be "
-                    'rated as "not cheese".',
+                    '    build-backend = "setuptools.build_meta"\n',
+                    "Specifying both a License-Expression and license classifiers is ambiguous, "
+                    "deprecated, and may be rejected by package indices.",
+                    "Check-manifest returned errors",
                 ],
             ),
         )
 
+    def test_skip_tests(self):
+        # Find all errors
+        all_errors = self._get_file_rating("lacking")[1]
+
+        fewer_errors = self._get_file_rating(
+            "lacking", skip_tests=["PythonRequiresVersion", "Description", "Summary", "Classifiers"]
+        )[1]
+
+        self.assertEqual(len(all_errors), 13)
+        # Errors have been skipped!
+        self.assertEqual(len(fewer_errors), 9)
+
     def test_pep517(self):
         rating = self._get_file_rating("pep517")
         self.assertEqual(
@@ -163,28 +203,25 @@ class RatingsTest(unittest.TestCase):
 
     def test_minimal(self):
         rating = self._get_file_rating("minimal")
-
         self.assertEqual(
             rating,
             (
                 2,
                 [
-                    "The package's description should be longer than 10 characters.",
-                    "The package's long_description is quite short.",
-                    "Your package does not have classifiers data.",
+                    "The package's Summary should be longer than 10 characters.",
+                    "The package's Description is quite short.",
+                    "Your package does not have classifier data.",
                     "The classifiers should specify what Python versions you support.",
-                    (
-                        "You should specify what Python versions you support with "
-                        "the 'requires-python'/'python_requires' metadata."
-                    ),
+                    "You should specify what Python versions you support with " "the 'Requires-Python' metadata.",
                     "Your package does not have keywords data.",
                     "Your package does not have author data.",
-                    "Your package does not have author_email data.",
+                    "Your package does not have author-email data.",
                     "Your package should have a 'url' field with a link to the project home page, or a "
                     "'project_urls' field, with a dictionary of links, or both.",
                     "Your package does neither have a license field nor any license classifiers.",
                     "Specifying a development status in the classifiers gives users "
                     "a hint of how stable your software is.",
+                    "Check-manifest returned errors",
                 ],
             ),
         )
@@ -197,21 +234,23 @@ class RatingsTest(unittest.TestCase):
             (
                 0,
                 [
-                    "The package had no description!",
-                    "The package's long_description is quite short.",
-                    "Your package does not have classifiers data.",
+                    "Your project does not have a pyproject.toml file, which is highly recommended.\n"
+                    "You probably want to create one with the following configuration:\n\n"
+                    "    [build-system]\n"
+                    '    requires = ["setuptools>=42"]\n'
+                    '    build-backend = "setuptools.build_meta"\n',
+                    "The package had no Summary!",
+                    "The package's Description is quite short.",
+                    "Your package does not have classifier data.",
                     "The classifiers should specify what Python versions you support.",
-                    (
-                        "You should specify what Python versions you support with "
-                        "the 'requires-python'/'python_requires' metadata."
-                    ),
+                    "You should specify what Python versions you support with " "the 'Requires-Python' metadata.",
                     "Your package does not have keywords data.",
                     "Your package does not have author data.",
-                    "Your package does not have author_email data.",
+                    "Your package does not have author-email data.",
                     "Your package should have a 'url' field with a link to the project home page, or a "
                     "'project_urls' field, with a dictionary of links, or both.",
                     "Your package does neither have a license field nor any license classifiers.",
-                    "Your long_description is not valid ReST: \n<string>:1: (WARNING/2) Inline literal "
+                    "Your Description is not valid ReST: \n<string>:1: (WARNING/2) Inline literal "
                     "start-string without end-string.",
                     "Specifying a development status in the classifiers gives users "
                     "a hint of how stable your software is.",
@@ -225,19 +264,16 @@ class RatingsTest(unittest.TestCase):
         self.assertEqual(
             rating,
             (
-                2,
+                3,
                 [
-                    "The package's description should be longer than 10 characters.",
-                    "The package's long_description is quite short.",
-                    "Your package does not have classifiers data.",
+                    "The package's Summary should be longer than 10 characters.",
+                    "The package's Description is quite short.",
+                    "Your package does not have classifier data.",
                     "The classifiers should specify what Python versions you support.",
-                    (
-                        "You should specify what Python versions you support with "
-                        "the 'requires-python'/'python_requires' metadata."
-                    ),
+                    "You should specify what Python versions you support with " "the 'Requires-Python' metadata.",
                     "Your package does not have keywords data.",
                     "Your package does not have author data.",
-                    "Your package does not have author_email data.",
+                    "Your package does not have author-email data.",
                     "Your package should have a 'url' field with a link to the project home page, or a "
                     "'project_urls' field, with a dictionary of links, or both.",
                     "Your package does neither have a license field nor any license classifiers.",
@@ -247,62 +283,41 @@ class RatingsTest(unittest.TestCase):
             ),
         )
 
+    def test_private_classifier(self):
+        rating = self._get_file_rating("private_classifier")
+        self.assertEqual(rating, (10, []))
+
     def test_markdown(self):
         # Markdown and text shouldn't get ReST errors
         testdata = COMPLETE.copy()
-        testdata["long_description"] = "# Broken ReST\n\n``Valid  Markdown\n"
-        testdata["long_description_content_type"] = "text/markdown"
-        rating = rate(testdata)
-        self.assertEqual(rating, (9, ["The package's long_description is quite short."]))
+        testdata["description"] = "# Broken ReST\n\n``Valid  Markdown\n"
+        testdata["description-content-type"] = "text/markdown"
 
-        testdata["long_description_content_type"] = "text/plain"
         rating = rate(testdata)
-        self.assertEqual(rating, (9, ["The package's long_description is quite short."]))
+        self.assertEqual(rating, (9, ["The package's Description is quite short."]))
+
+        testdata["description-content-type"] = "text/plain"
+        rating = rate(testdata)
+        self.assertEqual(rating, (9, ["The package's Description is quite short."]))
 
 
 class PyPITest(unittest.TestCase):
-
     maxDiff = None
 
     @unittest.mock.patch("xmlrpc.client.ServerProxy", proxystub)
     @unittest.mock.patch("pyroma.pypidata._get_project_data")
-    def test_distribute(self, projectdatamock):
-        datafile = resource_filename(__name__, os.path.join("testdata", "jsondata", "distribute.json"))
-        with open(datafile, "rt", encoding="UTF-8") as file:
-            projectdatamock.return_value = json.load(file)
-
-        proxystub.set_debug_context("distributedata.py", xmlrpclib.ServerProxy, False)
-
-        data = pypidata.get_data("distribute")
-        rating = rate(data)
-
-        # build + older versions of setuptools works, later setuptools does not, so
-        # we can get two different results here:
-        self.assertEqual(
-            rating,
-            (
-                7,
-                [
-                    "The classifiers should specify what minor versions of Python "
-                    "you support as well as what major version.",
-                    "You should have three or more owners of the project on PyPI.",
-                    "Your Cheese may have spoiled!! The only way to gather metadata from your package was to execute "
-                    "a patched setup.py. This indicates that your package is using very old packaging techniques, "
-                    "(or that your setup.py isn't executable at all), and Pyroma will soon regard that as a "
-                    "complete failure!\nPlease modernize your packaging! If it is already modern, this is a bug.",
-                ],
-            ),
-        )
-
-    @unittest.mock.patch("xmlrpc.client.ServerProxy", proxystub)
-    @unittest.mock.patch("pyroma.pypidata._get_project_data")
-    def test_complete(self, projectdatamock):
+    @unittest.mock.patch("requests.get")
+    def test_complete(self, requestmock, projectdatamock):
         datafile = resource_filename(__name__, os.path.join("testdata", "jsondata", "complete.json"))
-        with open(datafile, "rt", encoding="UTF-8") as file:
+        with open(datafile, encoding="UTF-8") as file:
             projectdatamock.return_value = json.load(file)
 
+        srcfile = resource_filename(__name__, os.path.join("testdata", "distributions", "complete-1.0.dev1.tar.gz"))
+        with open(srcfile, "rb") as file:
+            requestmock.return_value = unittest.mock.Mock()
+            requestmock.return_value.content = file.read()
+
         proxystub.set_debug_context("completedata.py", xmlrpclib.ServerProxy, False)
-
         data = pypidata.get_data("complete")
         rating = rate(data)
 
@@ -310,18 +325,17 @@ class PyPITest(unittest.TestCase):
 
 
 class ProjectDataTest(unittest.TestCase):
-
     maxDiff = None
 
     def test_complete(self):
         directory = resource_filename(__name__, os.path.join("testdata", "complete"))
 
         data = projectdata.get_data(directory)
+        del data["_path"]  # This changes, so I just ignore it
         self.assertEqual(data, COMPLETE)
 
 
 class DistroDataTest(unittest.TestCase):
-
     maxDiff = None
 
     def test_complete(self):
